@@ -117,8 +117,14 @@ impl rustls_client_config_builder {
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_new() -> *mut rustls_client_config_builder {
         ffi_panic_boundary! {
+            // Unwrap safety: *ring* default provider always has ciphersuites compatible with the
+            // default protocol versions.
+            let base =
+                ClientConfig::builder_with_provider(rustls::crypto::ring::default_provider().into())
+                    .with_safe_default_protocol_versions()
+                    .unwrap();
             let builder = ClientConfigBuilder {
-                base: rustls::ClientConfig::builder(),
+                base,
                 verifier: Arc::new(NoneVerifier),
                 cert_resolver: None,
                 alpn_protocols: vec![],
@@ -153,6 +159,9 @@ impl rustls_client_config_builder {
         builder_out: *mut *mut rustls_client_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
+            if builder_out.is_null() {
+                return NullParameter;
+            }
             let cipher_suites: &[*const rustls_supported_ciphersuite] =
                 try_slice!(cipher_suites, cipher_suites_len);
             let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
@@ -556,6 +565,9 @@ impl rustls_client_config {
         conn_out: *mut *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
+            if conn_out.is_null() {
+                return NullParameter;
+            }
             let server_name: &CStr = unsafe {
                 if server_name.is_null() {
                     return NullParameter;
