@@ -75,8 +75,14 @@ impl rustls_server_config_builder {
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_new() -> *mut rustls_server_config_builder {
         ffi_panic_boundary! {
+            // Unwrap safety: *ring* default provider always has ciphersuites compatible with the
+            // default protocol versions.
+            let base =
+                ServerConfig::builder_with_provider(rustls::crypto::ring::default_provider().into())
+                    .with_safe_default_protocol_versions()
+                    .unwrap();
             let builder = ServerConfigBuilder {
-                base: rustls::ServerConfig::builder(),
+                base,
                 verifier: WebPkiClientVerifier::no_client_auth(),
                 cert_resolver: None,
                 session_storage: None,
@@ -110,6 +116,9 @@ impl rustls_server_config_builder {
         builder_out: *mut *mut rustls_server_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
+            if builder_out.is_null() {
+                return NullParameter;
+            }
             let cipher_suites: &[*const rustls_supported_ciphersuite] =
                 try_slice!(cipher_suites, cipher_suites_len);
             let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
@@ -312,6 +321,9 @@ impl rustls_server_config {
         conn_out: *mut *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
+            if conn_out.is_null() {
+                return NullParameter;
+            }
             let config: Arc<ServerConfig> = try_clone_arc!(config);
 
             let server_connection = match ServerConnection::new(config) {
@@ -490,7 +502,7 @@ impl ResolvesServerCert for ClientHelloResolver {
         let mapped_sigs: Vec<u16> = client_hello
             .signature_schemes()
             .iter()
-            .map(|s| s.get_u16())
+            .map(|s| u16::from(*s))
             .collect();
         // Unwrap the Option. None becomes an empty slice.
         let alpn = match client_hello.alpn() {
